//-----------------------------------------------------------------------------------
// A simple code that test the MPS algorighm.
//-----------------------------------------------------------------------------------

#include "mps.h"
#include <iostream>
#include <cstdlib>
#include <climits>
#include <string>
#include <algorithm>
#include <iterator>
#include <random>
#include <vector>
#include <ogdf/fileformats/GraphIO.h>

using namespace std;


// functions defined in mps_test.cpp
int compute_removed_edge_size(string input_file, vector<int> post_order, int mutate_point);
void print_removed_edge_stats(string input_file, vector<int> post_order, int mutate_point);
vector<int> generate_post_order(string input_file);
vector<int> generate_guided_post_order(string input_file, vector<int> post_order);
vector<int> generate_mutated_post_order_at_x(string input_file, vector<int> post_order, int mutate_point);


void vector_printer(vector<int> state) {
    for (int i = 0; i < state.size(); ++i) {
        std::cout << state[i] << ",";
    }
    std::cout << std::endl;
}



void measure_removed_edges(string input_file, int k_max, int mutate_point) {
    // generate first state
    vector<int> state_old = generate_post_order(input_file);
    vector<int> state_new;


    int removed_old = compute_removed_edge_size(input_file, state_old, mutate_point);
    int removed_new;

    for (int k = 0; k < k_max; ++k) {
        std::cout << "first post order" << std::endl;
        vector_printer(state_old);

        // run mutation on canonical representation directly
        state_new = generate_mutated_post_order_at_x(input_file, state_old, mutate_point);
        // rotate output of mutated state to canonical representation
        state_new = generate_guided_post_order(input_file, state_new);
        // tree produced should be the same as tree from mutation
        removed_new = compute_removed_edge_size(input_file, state_new, mutate_point);

        std::cout << "mutated post order" << std::endl;
        vector_printer(state_new);

        
        // std::cout << "removed edges in old: " << removed_old << std::endl;
        // std::cout << "removed edges in new: " << removed_new << std::endl;
        print_removed_edge_stats(input_file, state_new, mutate_point);

    }

}

int get_graph_size(string input_file) {
    ogdf::Graph G;

    // utilize OGDF readGML
    if (!ogdf::GraphIO::read(G, input_file, ogdf::GraphIO::readGML)) {
        std::cerr << "Could not read " << input_file << ".gml" << std::endl;
    }

    return G.numberOfNodes();
}

//-----------------------------------------------------------------------------------
// Main function.
//-----------------------------------------------------------------------------------


int main(int argc, char* argv[]) {
    string input_file = argv[1];
    int k_max = std::stoi(argv[2]);
    int mutate_point = std::stoi(argv[3]);

    // generate order here
    measure_removed_edges(input_file, k_max, mutate_point);

	return 0;
}
